package info.bayro.baFwk.database;

import info.bayro.baFwk.exception.ServiceException;
import info.bayro.baFwk.resourcesReader.ResourceReader;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.Properties;

import org.apache.log4j.Logger;
import org.hsqldb.HsqlException;
import org.hsqldb.server.Server;


public class DBConnection {

	public static final Logger log = Logger.getLogger(DBConnection.class);
	private static Connection connection = null;
	private static Server server = null;
	private static String user = "";
	private static String password = "";
	private static String host = "";
	private static String database = "";
	private static String databasePath = "";


	private DBConnection() {}


	private static void startServer() throws ServiceException {
		if (DBConnection.server == null) {
			if (log.isInfoEnabled()) {
				log.info("start hsqldb-server");
			}

			String fileName = "bin/hsqldb.properties";

			try {
				readProperties(fileName);

				DBConnection.server = new Server();

				DBConnection.server.setAddress(host);
				DBConnection.server.setDatabaseName(0, database);
				DBConnection.server.setDatabasePath(0, databasePath);
				DBConnection.server.setTrace(false);
				DBConnection.server.setLogWriter(null);
				DBConnection.server.setErrWriter(null);
				DBConnection.server.setSilent(true);
				DBConnection.server.start();

				try {
					Class.forName("org.hsqldb.jdbc.JDBCDriver");
				} catch (ClassNotFoundException e) {
					throw new ServiceException("Can not find the hsql-jdbc-driver", e);
				}

			} catch (FileNotFoundException e) {
				throw new ServiceException("[0] file not found!", e, fileName);
			} catch (IOException e) {
				throw new ServiceException("IO Exception", e);
			} catch (HsqlException e) {
				throw new ServiceException("HSQL Exception", e);
			}

		}
	}


	private static void readProperties(String fileName) throws FileNotFoundException, IOException {
		if (log.isTraceEnabled()) {
			log.trace("Param: " + fileName);
		}

		Properties properties = ResourceReader.readFile(fileName);

		host = properties.getProperty("host", "localhost");
		database = properties.getProperty("database", "db");
		databasePath = properties.getProperty("databasePath", "~/hsqldb");
		user = properties.getProperty("user", "SA");
		password = properties.getProperty("password", "");
	}


	public static Connection getInstance() throws ServiceException, SQLException {
		if (log.isDebugEnabled()) {
			log.debug("get DB-Instance");
		}

		if (DBConnection.connection == null) {
			DBConnection.connection = getConnection();
		}

		return DBConnection.connection;
	}


	private static Connection getConnection() throws ServiceException, SQLException {
		if (log.isDebugEnabled()) {
			log.debug("get connection");
		}

		if (DBConnection.server == null) {
			startServer();
		}

		if (DBConnection.connection == null || DBConnection.connection.isClosed()) {
			String url = "jdbc:hsqldb:file:~/hsqldb/" + database;

			try {
				DBConnection.connection = DriverManager.getConnection(url, user, "");
			} catch (SQLException e) {
				throw new ServiceException("Can not connect to database.\nUrl: [0]\nUser: [1]\nPassword: [2]", e, url,
						user, password);
			}
		}

		return DBConnection.connection;
	}


	public static void stopServer() {
		if (log.isInfoEnabled()) {
			log.info("stop hsqldb");
		}

		if (DBConnection.server != null) {
			DBConnection.server.shutdown();
		}
	}


	public static void closeConnection() throws SQLException {
		if (log.isDebugEnabled()) {
			log.debug("close connection");
		}

		if (DBConnection.connection != null && DBConnection.connection.isClosed() == false) {
			DBConnection.connection.close();
		}
	}
}