package info.bayro.baFwk.eList;

import info.bayro.baFwk.valueObject.UpdateMsg;
import info.bayro.baFwk.valueObject.ValueObject;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Observable;

import org.apache.log4j.Logger;


public class EList<E extends ValueObject> extends Observable implements List<E> {

	public final Logger log = Logger.getLogger(this.getClass());

	private List<E> list = new ArrayList<E>();


	public boolean add(E e) {
		UpdateMsg msg = new UpdateMsg(e.getClass().getCanonicalName(), e.toString());
		fireUpdate(msg);
		return list.add(e);
	}


	public void add(int index, E element) {
		list.add(index, element);
	}


	public boolean addAll(Collection<? extends E> c) {
		return list.addAll(c);
	}


	public boolean addAll(int index, Collection<? extends E> c) {
		return list.addAll(index, c);
	}


	public void clear() {
		list.clear();
	}


	public boolean contains(Object o) {
		return list.contains(o);
	}


	public boolean containsAll(Collection<?> c) {
		return list.containsAll(c);
	}


	public boolean equals(Object o) {
		return list.equals(o);
	}


	public E get(int index) {
		return list.get(index);
	}


	public int hashCode() {
		return list.hashCode();
	}


	public int indexOf(Object o) {
		return list.indexOf(o);
	}


	public boolean isEmpty() {
		return list.isEmpty();
	}


	public Iterator<E> iterator() {
		return list.iterator();
	}


	public int lastIndexOf(Object o) {
		return list.lastIndexOf(o);
	}


	public ListIterator<E> listIterator() {
		return list.listIterator();
	}


	public ListIterator<E> listIterator(int index) {
		return list.listIterator(index);
	}


	public E remove(int index) {
		UpdateMsg msg = new UpdateMsg("index", index);
		fireUpdate(msg);
		return list.remove(index);
	}


	public boolean remove(Object o) {
		return list.remove(o);
	}


	public boolean removeAll(Collection<?> c) {
		return list.removeAll(c);
	}


	public boolean retainAll(Collection<?> c) {
		return list.retainAll(c);
	}


	public E set(int index, E element) {
		return list.set(index, element);
	}


	public int size() {
		return list.size();
	}


	public List<E> subList(int fromIndex, int toIndex) {
		return list.subList(fromIndex, toIndex);
	}


	public Object[] toArray() {
		return list.toArray();
	}


	public <T> T[] toArray(T[] a) {
		return list.toArray(a);
	}


	public void fireUpdate(UpdateMsg updateMsg) {
		if (log.isTraceEnabled()) {
			log.trace(updateMsg.toString());
		}

		setChanged();
		notifyObservers(updateMsg);
	}


	public String toString() {
		StringBuilder strBuilder = new StringBuilder();

		strBuilder.append(this.getClass().getCanonicalName());
		strBuilder.append(" [\n");

		for (ValueObject vo : list) {
			strBuilder.append(vo.toString()).append("\n");
		}

		strBuilder.append("]");

		return strBuilder.toString();
	}
}
