package info.bayro.baFwk.gui;

import info.bayro.baFwk.exception.ProcessingException;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.lang.annotation.Annotation;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.apache.log4j.Logger;


public abstract class EEventHandle implements ActionListener {

	public Logger log = Logger.getLogger(this.getClass());

	private ECommandHandle eCommandHandle;
	private EViewFactory viewFacotry;


	protected ECommandHandle getCommandHandle() {
		return eCommandHandle;
	}


	public void setCommandHandle(ECommandHandle eCommandHandle) {
		this.eCommandHandle = eCommandHandle;
	}


	protected void startView(String id, DataSelector ds) {
		if (log.isTraceEnabled()) {
			log.trace("param: id: " + id + ", dataSelector: " + ds);
		}
		viewFacotry.createView(id, ds);
	}


	protected void cancelView() {
		viewFacotry.cancelView();
	}


	public void actionPerformed(ActionEvent e) {
		String actionCommand = e.getActionCommand();
		callMethod(actionCommand);
	}


	private void callMethod(String actionCommand) {
		if (log.isTraceEnabled()) {
			log.trace("actionCommand: " + actionCommand);
		}
		
		Method[] declaredMethods = this.getClass().getDeclaredMethods();
		for (Method method : declaredMethods) {

			Annotation[] annotations = method.getAnnotations();
			for (Annotation annotation : annotations) {

				if (annotation instanceof EActionEvent) {
					EActionEvent eActionEvent = (EActionEvent) annotation;

					String event = eActionEvent.event();
					if (event.equals(actionCommand)) {

						try {
							
							if (log.isTraceEnabled()) {
								log.trace("method to invoke: " + method.getName());
							}
							
							method.invoke(this);
						} catch (IllegalArgumentException e1) {
							throw new ProcessingException("Illegal argument exception", e1);
						} catch (IllegalAccessException e1) {
							throw new ProcessingException("Illegal access exception", e1);
						} catch (InvocationTargetException e1) {
							throw new ProcessingException("Invocation target exception", e1);
						}
					}

				}
			}
		}
	}


	public void setViewFactoryRef(EViewFactory viewFactory) {
		this.viewFacotry = viewFactory;
	}


	public abstract void onStart(DataSelector ds);

}