package info.bayro.baFwk.gui;

import info.bayro.baFwk.exception.ServiceException;
import info.bayro.baFwk.guiElement.listener.EWindowListener;
import info.bayro.baFwk.resourcesReader.ResourceReader;

import java.awt.BorderLayout;
import java.awt.Container;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Properties;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;

import org.apache.log4j.Logger;


public class EFrame {

	public final Logger log = Logger.getLogger(this.getClass());

	private static JFrame jFrame = null;
	private JPanel jPanelCenter = null;
	private EPanelStack ePanelStack = null;


	public EFrame() throws FileNotFoundException, IOException, ServiceException {
		if (EFrame.jFrame == null) {
			init();
		}
	}


	private void setLookAndFeel() {
		if (log.isTraceEnabled()) {
			log.trace("set the look and feel");
		}

		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (UnsupportedLookAndFeelException e) {
			log.error(e);
		} catch (ClassNotFoundException e) {
			log.error(e);
		} catch (InstantiationException e) {
			log.error(e);
		} catch (IllegalAccessException e) {
			log.error(e);
		}
	}


	private void init() throws FileNotFoundException, IOException, ServiceException {
		if (log.isTraceEnabled()) {
			log.trace("init frame");
		}

		setLookAndFeel();

		Properties frameProperties = ResourceReader.readFile("bin/frame.properties");
		String title = frameProperties.getProperty("title", "Title");
		int width = Integer.valueOf(frameProperties.getProperty("width", "700"));
		int height = Integer.valueOf(frameProperties.getProperty("height", "600"));

		EFrame.jFrame = new JFrame();
		EFrame.jFrame.setTitle(title);
		EFrame.jFrame.setBounds(100, 100, width, height);
		EFrame.jFrame.addWindowListener(new EWindowListener());
		EFrame.jFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		EFrame.jFrame.setVisible(true);

		initPanels();

		if (log.isTraceEnabled()) {
			log.trace("set frame visibility to true!");
		}

	}


	private void initPanels() {
		if (log.isTraceEnabled()) {
			log.trace("create initials panels");
		}

		Container contentPane = EFrame.jFrame.getContentPane();

		JPanel jPanelNorth = new JPanel();
		contentPane.add(jPanelNorth, BorderLayout.NORTH);

		JPanel jPanelEast = new JPanel();
		contentPane.add(jPanelEast, BorderLayout.EAST);

		JPanel jPanelSouth = new JPanel();
		contentPane.add(jPanelSouth, BorderLayout.SOUTH);

		JPanel jPanelWest = new JPanel();
		contentPane.add(jPanelWest, BorderLayout.WEST);

		jPanelCenter = new JPanel(new BorderLayout());
		contentPane.add(jPanelCenter, BorderLayout.CENTER);

		JLabel jLabelCenter = new JLabel("Center");
		jPanelCenter.add(jLabelCenter);

		if (log.isTraceEnabled()) {
			log.trace("add the centerLabel to the panelstack");
		}

		ePanelStack = new EPanelStack();
		ePanelStack.getStackPanel().add(jLabelCenter);
	}


	/**
	 * add/show the new panel to the view
	 * 
	 * @param jComponent
	 *            new panel to show
	 */
	public void add(JComponent jComponent) {
		if (log.isTraceEnabled()) {
			log.trace("add a new panel");
		}

		// add the new panel to the stack
		ePanelStack.getStackPanel().add(jComponent);

		// add the new panel to the view
		jPanelCenter.removeAll();

		// add the last element
		jPanelCenter.add(jComponent, BorderLayout.CENTER);

		// refresh the panel
		jPanelCenter.repaint();
	}


	/**
	 * delete the current panel from the stack
	 */
	public void delCurrentComponent() {
		if (log.isTraceEnabled()) {
			log.trace("delete the current panel");
		}

		int size = ePanelStack.getStackPanel().size();
		ePanelStack.getStackPanel().remove(size - 1);

		// get the last element from the stack after deleting
		JComponent jComponent = ePanelStack.getStackPanel().get(size - 2);

		// delete the current content
		jPanelCenter.removeAll();

		// add the last element
		jPanelCenter.add(jComponent, BorderLayout.CENTER);

		// refresh the panel
		jPanelCenter.repaint();
	}
}