package info.bayro.baFwk.guiElement;

import info.bayro.baFwk.eList.EList;
import info.bayro.baFwk.exception.ProcessingException;
import info.bayro.baFwk.gui.DataContainer;
import info.bayro.baFwk.valueObject.ValueObject;

import java.util.Observable;

import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;


public class ETable extends ExtendedElement {

	private JTable jTable;
	private ETableModel model;

	private EList<?> eList;


	@Override
	protected void init() {
		model = new ETableModel();
		jTable = new JTable(model);

		jTable.setEnabled(true);
		jTable.setFillsViewportHeight(true);
		jTable.setShowGrid(true);
		jTable.setCellSelectionEnabled(false);
		jTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		jTable.setRowSelectionAllowed(true);

		jComponent = new JScrollPane(jTable);
	}


	public void addColumn(ETableColumn eTableColumn) {
		model.addColumn(eTableColumn.getTableColumn().getHeaderValue().toString());
	}


	public void setAttribute(DataContainer dataContainer, String fieldPath) {

		if (dataContainer != null &&
				fieldPath != null &&
				Boolean.FALSE.equals(fieldPath.trim().isEmpty())) {

			this.fieldName = fieldPath;
			this.valueObject = dataContainer;

			String[] fieldSplit = fieldPath.split("\\.");
			eList = getEListRef(dataContainer, fieldSplit);
			
			if (eList != null){
				for (Object object : eList) {
					ValueObject valueObject = (ValueObject) object;
					Object[] values = valueObject.getValues();
					model.addData(values);
				}
				
				eList.addObserver(this);
			}

		}
	}


	private EList<?> getEListRef(ValueObject valueObject, String[] fieldPath) {

		try {

			String type = valueObject.getType(fieldPath[0]);
			Class<?> clazz = Class.forName(type);

			if (EList.class.isAssignableFrom(clazz)) {
				return (EList<?>) valueObject.getValue(fieldPath[0]);
				
			} else  if (ValueObject.class.isAssignableFrom(clazz)) {
				ValueObject vo = (ValueObject) valueObject.getValue(fieldPath[0]);
				String[] field = new String[fieldPath.length - 1];

				for (int i = 1; i < fieldPath.length; i++) {
					field[i - 1] = fieldPath[i];
				}

				return getEListRef(vo, field);
			}

			return null;

		} catch (SecurityException e) {
			String msg = "Security exception!\nDataContainer: [0]\nField: [1]";
			throw new ProcessingException(msg, e, valueObject.getClass(), fieldPath);

		} catch (NoSuchFieldException e) {
			String msg = "No field found!\nDataContainer: [0]\nField: [1]";
			throw new ProcessingException(msg, e, valueObject.getClass(), fieldPath);

		} catch (IllegalArgumentException e) {
			String msg = "Illegal argument exception!\nDataContainer: [0]\nField: [1]";
			throw new ProcessingException(msg, e, valueObject.getClass(), fieldPath);

		} catch (IllegalAccessException e) {
			String msg = "Illegal access exception!\nDataContainer: [0]\nField: [1]";
			throw new ProcessingException(msg, e, valueObject.getClass(), fieldPath);

		} catch (ClassNotFoundException e) {
			String msg = "Class not found exception!\nDataContainer: [0]\nField: [1]";
			throw new ProcessingException(msg, e, valueObject.getClass(), fieldPath);
		}
	}


	public void setEnabled(boolean value) {
		jTable.setEnabled(value);
	}


	public void setActionCommand(String command) {
		model.setActionCommand(command);
	}


	@Override
	public void update(Observable observable, Object msg) {

		if (observable.getClass().equals(eList.getClass())) {

			String[] fieldSplit = fieldName.split("\\.");
			eList = getEListRef(valueObject, fieldSplit);
			
			if (eList != null){
				model.removeAllRows();
				
				for (Object object : eList) {
					ValueObject valueObject = (ValueObject) object;
					Object[] values = valueObject.getValues();
					model.addData(values);
				}
			}
		}
	}

}