package info.bayro.baFwk.guiElement;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.util.Observable;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import info.bayro.baFwk.exception.ProcessingException;
import info.bayro.baFwk.gui.DataContainer;
import info.bayro.baFwk.guiElement.listener.EDocumentVerifier;
import info.bayro.baFwk.guiElement.listener.EFocusListener;
import info.bayro.baFwk.guiElement.listener.ETextFieldVerifier;
import info.bayro.baFwk.valueObject.UpdateMsg;
import info.bayro.baFwk.valueObject.ValueChecker;
import info.bayro.baFwk.valueObject.ValueDescriptor;
import info.bayro.baFwk.valueObject.ValueFieldType;
import info.bayro.baFwk.valueObject.ValueObject;


public class ETextField extends ExtendedElement {

	private JTextField jTextField;
	private JLabel jLabel;


	protected void init() {
		jLabel = new JLabel();

		jTextField = new JTextField(10);
		jTextField.setBorder(EElementGlobals._BORDER_GRAY);
		jTextField.setBackground(EElementGlobals._COLOR_WHITE);
		
		jComponent = new JPanel();
		jComponent.setLayout(new GridBagLayout());

		GridBagConstraints gridJLabel = new GridBagConstraints();
		gridJLabel.fill = GridBagConstraints.NONE;
		gridJLabel.gridx = 0;
		gridJLabel.gridy = 0;
		gridJLabel.insets = new Insets(0, 5, 0, 5);
		gridJLabel.anchor = GridBagConstraints.WEST;
		jComponent.add(jLabel, gridJLabel);

		GridBagConstraints gridJTextField = new GridBagConstraints();
		gridJTextField.fill = GridBagConstraints.NONE;
		gridJTextField.gridx = 1;
		gridJTextField.gridy = 0;
		gridJTextField.weightx = gridJTextField.weighty = 1;
		gridJTextField.anchor = GridBagConstraints.WEST;
		gridJTextField.insets = new Insets(0, 5, 0, 5);
		jComponent.add(jTextField, gridJTextField);
	}


	public void setColumns(int column) {
		jTextField.setColumns(column);
	}


	private void setDescriptor(String fieldName) throws SecurityException,
			NoSuchFieldException {

		ValueDescriptor valueDescriptor = ValueChecker.getValueDescriptor(valueObject, fieldName);
		if (valueDescriptor != null) {
			ValueFieldType fieldType = valueDescriptor.fieldType();
			int maxSize = valueDescriptor.maxSize();
			int fieldDigits = valueDescriptor.fieldDigits();

			// check the input
			EDocumentVerifier doc = new EDocumentVerifier(jTextField, fieldType, maxSize, fieldDigits);
			jTextField.setDocument(doc);

			// check the input after lost of focus
			ETextFieldVerifier eVerifier = new ETextFieldVerifier(fieldType, maxSize, fieldDigits);
			jTextField.setInputVerifier(eVerifier);

			// assign the input to the valueObject and fireUpdate
			EFocusListener eFocusListener = new EFocusListener(valueObject, fieldName);
			jTextField.addFocusListener(eFocusListener);
		}
	}


	public void setAttribute(DataContainer dataContainer, String fieldPath) {

		if (dataContainer != null &&
				fieldPath != null &&
				Boolean.FALSE.equals(fieldPath.trim().isEmpty())) {

			try {

				String[] fieldSplit = fieldPath.split("\\.");

				this.valueObject = identifyLastValueObject(dataContainer, fieldSplit);
				this.fieldName = identifyLastField(fieldSplit);

				setDescriptor(fieldName);

				initField(fieldName);
				
				this.valueObject.addObserver(this);

			} catch (SecurityException e) {
				String msg = "Security exception!\nDataContainer: [0]\nField: [1]";
				throw new ProcessingException(msg, e, dataContainer.getClass(), fieldPath);

			} catch (NoSuchFieldException e) {
				String msg = "No field found!\nDataContainer: [0]\nField: [1]";
				throw new ProcessingException(msg, e, dataContainer.getClass(), fieldPath);

			} catch (IllegalArgumentException e) {
				String msg = "Illegal argument exception!\nDataContainer: [0]\nField: [1]";
				throw new ProcessingException(msg, e, dataContainer.getClass(), fieldPath);

			} catch (IllegalAccessException e) {
				String msg = "Illegal access exception!\nDataContainer: [0]\nField: [1]";
				throw new ProcessingException(msg, e, dataContainer.getClass(), fieldPath);

			} catch (ClassNotFoundException e) {
				String msg = "Class not found exception!\nDataContainer: [0]\nField: [1]";
				throw new ProcessingException(msg, e, dataContainer.getClass(), fieldPath);
			}
		}
	}


	private ValueObject identifyLastValueObject(ValueObject valueObject, String[] fieldPath) throws SecurityException,
			NoSuchFieldException, ClassNotFoundException, IllegalArgumentException, IllegalAccessException {

		if (fieldPath.length == 1) {
			return valueObject;
		}

		String type = valueObject.getType(fieldPath[0]);
		Class<?> clazz = Class.forName(type);

		if (ValueObject.class.isAssignableFrom(clazz)) {

			ValueObject vo = (ValueObject) valueObject.getValue(fieldPath[0]);
			String[] field = new String[fieldPath.length - 1];

			for (int i = 1; i < fieldPath.length; i++) {
				field[i-1] = fieldPath[i];
			}

			return identifyLastValueObject(vo, field);
		}

		return null;
	}


	private String identifyLastField(String[] fieldPath) {
		int length = fieldPath.length;
		return fieldPath[length - 1];
	}


	private void initField(String fieldName) throws SecurityException,
			NoSuchFieldException, IllegalArgumentException, IllegalAccessException, ClassNotFoundException {

		String value = String.valueOf(valueObject.getValue(fieldName));

		if (value != null && !"null".equals(value) && value.trim().isEmpty() == false) {

			String clazzName = valueObject.getType(fieldName);
			Class<?> clazz = Class.forName(clazzName);

			if (Number.class.isAssignableFrom(clazz)) {
				value = value.replace(".", ",");
			}

			jTextField.setText(value);
		}

	}


	public void setLabel(String label) {
		jLabel.setText(label);
	}


	public void setEditable(boolean value) {
		jTextField.setEditable(value);

		if (value == false) {
			jTextField.setBackground(EElementGlobals._COLOR_DISABLE);
		} else {
			jTextField.setBackground(EElementGlobals._COLOR_WHITE);
		}
	}


	@Override
	public void update(Observable observable, Object msg) {

		if (observable.getClass().equals(valueObject.getClass()) && msg != null) {

			UpdateMsg updateMsg = (UpdateMsg) msg;

			if (fieldName.equals(updateMsg.getField())) {
				if (updateMsg.getValue() != null && !updateMsg.getValue().trim().isEmpty()) {
					jTextField.setText(updateMsg.getValue().trim());
				} else {
					jTextField.setText("");
				}
			}
		}
	}
}