package info.bayro.baFwk.guiElement.listener;

import java.util.Set;

import javax.swing.JTextField;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;

import info.bayro.baFwk.guiElement.EElementGlobals;
import info.bayro.baFwk.valueObject.ValueChecker;
import info.bayro.baFwk.valueObject.ValueFieldType;


public class EDocumentVerifier extends PlainDocument {

	private static final long serialVersionUID = 4661678066950774388L;

	private final Set<Character> _SYMBOL;

	private ValueFieldType fieldType = ValueFieldType._FIELD_ALPHANUM;
	private int fieldSize;
	private int fieldDigits;
	private JTextField jTextField;


	public EDocumentVerifier(JTextField jTextField, ValueFieldType fieldType, int fieldSize, int fieldDigits) {
		this.jTextField = jTextField;
		this.fieldType = fieldType;
		this.fieldSize = fieldSize;
		this.fieldDigits = fieldDigits;

		_SYMBOL = ValueChecker.createSet(fieldType, fieldDigits);
	}


	public void insertString(int offs, String str, AttributeSet a) throws BadLocationException {
		if (str.trim().isEmpty()) {
			return;
		}

		// nur valide character erlauben
		for (int i = 0; i < str.length(); i++) {
			char character = str.charAt(i);

			if (!_SYMBOL.contains(character)) {
				return;
			}
		}

		String text = getText(0, getLength());

		// zukünftigen string simulieren
		String subStr1 = text.substring(0, offs);
		String subStr2 = text.substring(offs);
		String possibleStr = subStr1 + str + subStr2;

		// wenn alphanumerisch nur länge prüfen
		if (fieldType == ValueFieldType._FIELD_ALPHANUM) {

			// länge prüfen
			if (possibleStr.length() > fieldSize) {
				return;
			}

		} else {
			String[] split = possibleStr.split(",");

			for (int i = 0; i < str.length(); i++) {
				char character = str.charAt(i);

				int possibleLength = fieldSize;

				if (fieldType == ValueFieldType._FIELD_SIGNED ||
						fieldType == ValueFieldType._FIELD_UNSIGNED) {

					if (possibleStr.contains("-")) {
						possibleLength += 1;
					}

					// vorzeichen nur einmal erlauben
					if (text.contains("-") && character == '-') {
						return;
					}

					// vorkomma-länge prüfen
					if ((character != '-' || character != ',') &&
							split.length > 0 &&
							split[0].length() > possibleLength) {
						return;
					}

					// vorzeichen nur am anfang erlauben
					if (offs != 0 && character == '-') {
						return;
					}
				}

				// komma nur erlauben wenn nachkomma angegeben werden darf
				if (fieldDigits > 0) {

					if (text.contains(",") && character == ',') {
						return;
					}

					int possibleDigits = fieldDigits;
					if (possibleStr.contains(",")) {
						possibleDigits += fieldDigits + 1;
					}

					// nachkomma-länge prüfen
					if (split.length >= 2 && split[1].length() > possibleDigits) {
						return;
					}

					// nach dem komma nur anzahl digits-zeichen erlauben
					if (split.length == 2 && split[1].length() > fieldDigits) {
						return;
					}
				}
			}
		}

		if (jTextField.isEditable()) {
			jTextField.setBackground(EElementGlobals._COLOR_WHITE);
		}

		super.insertString(offs, str, a);
	}

}