package info.bayro.baFwk.guiElement.listener;

import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;

import javax.swing.JTextField;

import info.bayro.baFwk.exception.ProcessingException;
import info.bayro.baFwk.valueObject.ValueObject;
import info.bayro.baFwk.valueObject.UpdateMsg;


public class EFocusListener implements FocusListener {

	private ValueObject valueObject;
	private String fieldName;


	public EFocusListener(ValueObject valueObject, String field) {
		this.valueObject = valueObject;
		this.fieldName = field;
	}


	public void focusGained(FocusEvent arg0) {}


	public void focusLost(FocusEvent arg0) {
		try {
			Field field = valueObject.getClass().getDeclaredField(fieldName);
			field.setAccessible(true);

			String clazzName = valueObject.getType(fieldName);

			Class<?> clazz = Class.forName(clazzName);
			Constructor<?> constructor = clazz.getConstructor(String.class);

			String text = ((JTextField) arg0.getComponent()).getText().trim();
			String replace = text;
			if (Number.class.isAssignableFrom(clazz)) {
				replace = text.replace(",", ".");
			}

			Object cast = null;
			if (replace != null && !replace.isEmpty()) {
				cast = constructor.newInstance(replace);
			}

			field.set(valueObject, cast);

			UpdateMsg updateMsg = new UpdateMsg(fieldName, text);
			valueObject.getClass().getMethod("fireUpdate", UpdateMsg.class).invoke(valueObject, updateMsg);

		} catch (InvocationTargetException e) {
			String msg = "Invocation exception";
			throw new ProcessingException(msg, e);

		} catch (SecurityException e) {
			String msg = "Security exception: [0]";
			throw new ProcessingException(msg, e, fieldName);

		} catch (NoSuchFieldException e) {
			String msg = "No such field exception: [0]";
			throw new ProcessingException(msg, e, fieldName);

		} catch (IllegalArgumentException e) {
			String msg = "Illegal argument exception: [0]";
			throw new ProcessingException(msg, e, fieldName);

		} catch (IllegalAccessException e) {
			String msg = "Illegal access exception: [0]";
			throw new ProcessingException(msg, e, fieldName);

		} catch (ClassNotFoundException e) {
			String msg = "Class not found exception: [0]";
			throw new ProcessingException(msg, e, fieldName);

		} catch (NoSuchMethodException e) {
			String msg = "No such method exception";
			throw new ProcessingException(msg, e);

		} catch (InstantiationException e) {
			String msg = "Instantiation exception";
			throw new ProcessingException(msg, e);
		}
	}
}