package info.bayro.baFwk.valueObject;

import java.lang.reflect.Field;
import java.util.Observable;

import org.apache.log4j.Logger;


public abstract class ValueObject extends Observable implements Cloneable {

	public static final Logger log = Logger.getLogger(ValueObject.class);


	/**
	 * Call the observer with the Update-Msg object.
	 * 
	 * Don't change the name of the method! It is also in use with
	 * java-reflection!
	 * 
	 * @param updateMsg
	 *            UpdateMsg-Object
	 */
	public void fireUpdate(UpdateMsg updateMsg) {
		if (log.isTraceEnabled()) {
			log.trace(updateMsg.toString());
		}

		setChanged();
		notifyObservers(updateMsg);
	}


	public Object getValue(String fieldName) throws
			SecurityException, NoSuchFieldException, IllegalArgumentException, IllegalAccessException {

		Field field = this.getClass().getDeclaredField(fieldName);
		field.setAccessible(true);

		return field.get(this);
	}


	public String getType(String fieldName) throws SecurityException, NoSuchFieldException {
		Field field = this.getClass().getDeclaredField(fieldName);
		field.setAccessible(true);

		return field.getType().getCanonicalName();
	}


	public Field getField(String field) throws SecurityException, NoSuchFieldException {
		return this.getClass().getDeclaredField(field);
	}


	/**
	 * This method returns an array of objects, which includes all the
	 * attributes and their values. Will be needed for jtables.
	 * 
	 * @return [] object
	 */
	public Object[] getValues() {
		Field[] declaredFields = this.getClass().getDeclaredFields();

		Object[] objArr = new Object[declaredFields.length];

		int i = 0;
		for (Field field : declaredFields) {
			field.setAccessible(true);

			try {
				objArr[i] = field.get(this);
				i++;
			} catch (IllegalArgumentException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			}
		}

		return objArr;
	}


	@Override
	public String toString() {
		StringBuilder strBuilder = new StringBuilder();

		strBuilder.append(this.getClass().getName());
		strBuilder.append(" [\n");

		Field[] declaredFields = this.getClass().getDeclaredFields();
		for (Field field : declaredFields) {
			field.setAccessible(true);

			try {
				strBuilder.append(field.getName());
				strBuilder.append(" = ");
				strBuilder.append(field.get(this));
				strBuilder.append("\n");
			} catch (IllegalArgumentException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			} catch (SecurityException e) {
				e.printStackTrace();
			}
		}

		strBuilder.append("]");

		return strBuilder.toString();
	}

}