package info.bayro.baFwk.junit.valueObject;

import org.junit.Test;

import static org.junit.Assert.*;

import info.bayro.baFwk.datatypes.Betrag;
import info.bayro.baFwk.datatypes.KundenNummer;
import info.bayro.baFwk.exception.InvalidValueException;


public class TestValueObject {

	@Test
	public void createBetragValue() throws InvalidValueException {
		assertNotNull(new Betrag("1234567890.12"));
	}


	@Test(expected = InvalidValueException.class)
	public void expectBetragStrException() throws InvalidValueException {
		new Betrag("1234567890a.12");
	}


	@Test()
	public void createKundenNr() throws InvalidValueException {
		assertNotNull(new KundenNummer("1234567890"));
	}


	@Test(expected = InvalidValueException.class)
	public void expectKundenNrStrException() throws InvalidValueException {
		new KundenNummer("1234567890a");
	}


	@Test(expected = InvalidValueException.class)
	public void expectTestVoNrKundeLengthException() throws InvalidValueException {
		TestVO testVo = new TestVO();
		testVo.setNrKunde(new KundenNummer("1234567890000"));
	}


	@Test(expected = InvalidValueException.class)
	public void expectTestVoNrKundeDigitException() throws InvalidValueException {
		TestVO testVo = new TestVO();
		testVo.setNrKunde(new KundenNummer("1234567890.12"));
	}


	@Test(expected = InvalidValueException.class)
	public void expectTestVoBetragLengthException() throws InvalidValueException {
		TestVO testVo = new TestVO();
		testVo.setBetrKaufpreis(new Betrag("123456789000.12"));
	}


	@Test(expected = InvalidValueException.class)
	public void expectTestVoBetragDigitException() throws InvalidValueException {
		TestVO testVo = new TestVO();
		testVo.setBetrKaufpreis(new Betrag("1234567890.123"));
	}


	@Test(expected = InvalidValueException.class)
	public void expectTestVoBezStrLengthException() throws InvalidValueException {
		TestVO testVo = new TestVO();
		testVo.setBezStr("abcdefghijklmnox");
	}


	@Test(expected = InvalidValueException.class)
	public void expectTestVoBezStrCharException() throws InvalidValueException {
		TestVO testVo = new TestVO();
		testVo.setBezStr("123456789a!%&'#%");
	}


	@Test
	public void expectTestVoBezStrEmptyStr() throws InvalidValueException {
		TestVO testVo = new TestVO();
		testVo.setBezStr("");

		assertNotNull(testVo);
	}


	@Test(expected = InvalidValueException.class)
	public void expectTestVoBezNameDescriptorException() throws InvalidValueException {
		TestVO testVo = new TestVO();
		testVo.setBezName("abc");
	}


	@Test
	public void expectTestVoBezStrNull() throws InvalidValueException {
		TestVO testVo = new TestVO();
		testVo.setBezStr(null);

		assertNotNull(testVo);
	}


	@Test
	public void createTestVo() throws InvalidValueException {
		TestVO testVo = new TestVO();

		testVo.setBetrKaufpreis(new Betrag("1234567890.12"));
		testVo.setNrKunde(new KundenNummer("1234567890"));
		testVo.setBezStr("Hallo Str. 12");

		assertNotNull(testVo);
	}

}
